# encoding: utf-8

module AMQP
  # amqp gem version. Not to be confused with the AMQP protocol version
  # it implements. For that, see {AMQ::Protocol::VERSION}
  #
  # @see AMQ::Protocol::VERSION
  # @return [String] AMQP gem version
  VERSION = '0.9.5'
end
